/*-
 * crypt: simple encryption routine for passwords.
 *
 * crypt() is a (hopefully) one-way cipher.  There is no way to
 * retrieve the original password given the encrypted version.
 * crypt accepts two arguments - pw & salt.
 * Pw is the plaintext password to be encrypted.
 * Salt is a small string (2 characters) used to permute the
 * encryption engine to frustrate attempts to 'crack' a password.
 * A good "salt" is the bottom 12 bits of the current time (in seconds)
 * encoded  as ascii.
 *
 * To create a new password, pass a "salt" and the plaintext version to
 * crypt();
 *
 * To later verify a password, read back the encrypted version, and pass
 * it as a Salt to crypt, along with the plain text the user just entered.
 *
 * the output of crypt should be the same string as the encrypted version.
 *
 */


static
bin2ascii(short x)
{
	if (x <= 9)
		return x + '0';
	x -= 9;
	if (x <= 26)
		return x + 'A';
	x -= 26;
	return x + 'a';
}


char           *
crypt(char *pw, char *salt)
{
	static char     buf[14];
	char            bits[67];
	int             i;
	int             j, rot;

	for (i = 0; i < sizeof bits / sizeof bits[0]; i++)
		bits[i] = 0;
	if (salt[1] == 0)
		salt[1] = salt[0];
	rot = (salt[1] * 4 - salt[0]) % 128;
	for (i = 0; *pw && i < 8; i++) {
		for (j = 0; j < 7; j++)
			bits[i + j * 8] = (*pw & (1 << j) ? 1 : 0);
		bits[i + 56] = (salt[i / 4] & (1 << (i % 4)) ? 1 : 0);
		pw++;
	}
	bits[64] = (salt[0] & 1 ? 1 : 0);
	bits[65] = (salt[1] & 1 ? 1 : 0);
	bits[66] = (rot & 1 ? 1 : 0);
	while (rot--) {
		for (i = 65; i >= 0; i--)
			bits[i + 1] = bits[i];
		bits[0] = bits[66];
	}
	for (i = 0; i < 12; i++) {
		buf[i + 2] = 0;
		for (j = 0; j < 6; j++)
			buf[i + 2] |= (bits[i * 6 + j] ? (1 << j) : 0);
		buf[i + 2] = bin2ascii(buf[i + 2]);
	}
	buf[0] = salt[0];
	buf[1] = salt[1];
	buf[13] = '\0';
	return buf;
}
